unit uDrawPictureSupport;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Clipbrd,
  Dialogs, GR32_Image, GR32_Layers, GR32, GR32_Resamplers, jpeg, StdCtrls;

type
  TPictureLayer = class(TRubberbandLayer)
  private
    FMaxHeigth: Integer;
    FMaxWidth : Integer;
    FProportion : Double;
    FParentImage : TImage32;
    FMasterAlpha: Integer;
    procedure CalculateMaxSize(AMaxPixelSize : Integer);
    procedure SetMasterAlpha(const Value: Integer);
    procedure RescaleToMaxSizeIfNeed();
  protected
    procedure DoResizing(var OldLocation, NewLocation: TFloatRect; DragState: TDragState; Shift: TShiftState); override;
  published
  public
    procedure SaveBitmapToStream(var ABitmapRect : TRect; AStream : TStream);
    procedure DrawBitmap(ADestBitmap : TBitmap32);
    property MasterAlpha : Integer read FMasterAlpha write SetMasterAlpha;
    class function CreatePictureLayer(AImage32: TImage32; AFileName: String;
      AMaxPixelSize: Integer): TPictureLayer;
    destructor Destroy(); override;
  end;

implementation

uses Math;
procedure TPictureLayer.CalculateMaxSize(AMaxPixelSize: Integer);
begin
  FMaxWidth := Min(Trunc(Sqrt(FProportion * AMaxPixelSize)), 550); //550 - max image width in commfort
  FMaxHeigth := Trunc(FMaxWidth / FProportion);
end;

class function TPictureLayer.CreatePictureLayer(AImage32: TImage32; AFileName: String;
  AMaxPixelSize: Integer): TPictureLayer;
var PictureWidth, PictureHeight : Integer;
    PLeftTop, PRightBotom: TPoint;
begin
  AImage32.BeginUpdate();
  try
    Result := TPictureLayer.Create(AImage32.Layers);
    with Result do
    try
      ChildLayer := TBitmapLayer.Create(AImage32.Layers);
      FParentImage := AImage32;
      ChildLayer.BringToFront();

      LayerOptions := LOB_VISIBLE or LOB_MOUSE_EVENTS or LOB_NO_UPDATE;
      Cursor := crSizeAll;
      with TBitmapLayer(ChildLayer).Bitmap do
      begin
        if Length(AFileName) > 0 then
          LoadFromFile(AFileName)
        else
          Assign(Clipboard);

        DrawMode := dmBlend;
        Resampler := TLinearResampler.Create(TBitmapLayer(ChildLayer).Bitmap);
        FProportion := Width / Height;
        CalculateMaxSize(AMaxPixelSize);
        with AImage32.GetViewportRect do
          PLeftTop := AImage32.ControlToBitmap(GR32.Point((Right + Left) div 2, (Top + Bottom) div 2));
        PictureWidth := Min(Width, FMaxWidth);
        PictureHeight := Trunc(PictureWidth / FProportion);
        if (PictureWidth < 5) or (PictureHeight < 5) or (FProportion < (1/5)) or (FProportion > 5) then
          raise Exception.Create('bad image');
        PLeftTop.X := PLeftTop.X - PictureWidth div 2;
        PLeftTop.Y := PLeftTop.Y - PictureHeight div 2;
        PRightBotom.X := PLeftTop.X + PictureWidth;
        PRightBotom.Y := PLeftTop.Y + PictureHeight;
      end;

      Location := GR32.FloatRect(PLeftTop.X, PLeftTop.Y, PRightBotom.X, PRightBotom.Y);
      Scaled := True;
      RescaleToMaxSizeIfNeed();
        FMaxWidth := 800;
        FMaxHeigth := Trunc(FMaxWidth / FProportion);
      BringToFront();
    except
      FreeAndNil(Result);
    end;
  finally
    AImage32.EndUpdate();
    AImage32.Changed();
  end;
end;

destructor TPictureLayer.Destroy;
begin
  if Assigned(ChildLayer) then
    ChildLayer.Free();
  inherited;
end;

procedure TPictureLayer.DoResizing(var OldLocation, NewLocation: TFloatRect;
  DragState: TDragState; Shift: TShiftState);
const MIN_BORDER_OFFSET = 10;
var  TmpSingle : Single;
begin
  inherited;
  with NewLocation do
  case DragState of
    dsSizeT, dsSizeTR :
    begin
      Top := Bottom - Min(FMaxHeigth, Bottom - Top);
      Right := Left + (Bottom - Top) * FProportion;
    end;
    dsSizeTL :
    begin
      Top := Bottom - Min(FMaxHeigth, Bottom - Top);
      Left := Right - (Bottom - Top) * FProportion;
    end;
    dsSizeB:
    begin
      Bottom := Top + Min(FMaxHeigth, Bottom - Top);
      Right := Left + (Bottom - Top) * FProportion;
    end;
    dsSizeBL:
    begin
      Bottom := Top + Min(FMaxHeigth, Bottom - Top);
      Left := Right - (Bottom - Top) * FProportion;
    end;
    dsSizeL:
    begin
      Left := Right - Min(FMaxWidth, Right - Left);
      Bottom := Top + (Right - Left) / FProportion;
    end;
    dsSizeR, dsSizeBR:
    begin
      Right := Left + Min(FMaxWidth, Right - Left);
      Bottom := Top + (Right - Left) / FProportion;
    end;
  end;
//  check for minimal size
//  if (DragState in [dsSizeL, dsSizeT, dsSizeR, dsSizeB,
//                   dsSizeTL, dsSizeTR, dsSizeBL, dsSizeBR]) and
//     (((NewLocation.Right - NewLocation.Left) < 30) or
//      ((NewLocation.Bottom - NewLocation.Top) < 30)) then
//    NewLocation := OldLocation;
  //check to sure position out of image
  if DragState = dsMove then
  with NewLocation do
  begin
    if (Bottom < MIN_BORDER_OFFSET) then
    begin
      Top := Top + MIN_BORDER_OFFSET - Bottom;
      Bottom := MIN_BORDER_OFFSET;
    end
    else
    if (Top > (FParentImage.Height - MIN_BORDER_OFFSET)) then
    begin
      TmpSingle := Bottom - Top;
      Top := FParentImage.Height - MIN_BORDER_OFFSET;
      Bottom := Top + TmpSingle;
    end;
    if (Right < MIN_BORDER_OFFSET) then
    begin
      Left := Left + MIN_BORDER_OFFSET - Right;
      Right := MIN_BORDER_OFFSET;
    end
    else
    if (Left > (FParentImage.Width - MIN_BORDER_OFFSET)) then
    begin
      TmpSingle := Right - Left;
      Left := FParentImage.Width - MIN_BORDER_OFFSET;
      Right := Left + TmpSingle;
    end;
  end;
end;

procedure TPictureLayer.DrawBitmap(ADestBitmap: TBitmap32);
begin
  ADestBitmap.DrawMode := dmBlend;
  ADestBitmap.MasterAlpha := Self.MasterAlpha;
  ADestBitmap.Resampler := TLinearResampler.Create();
  ADestBitmap.Draw(MakeRect(TBitmapLayer(ChildLayer).Location),
    Rect(0, 0, TBitmapLayer(ChildLayer).Bitmap.Width, TBitmapLayer(ChildLayer).Bitmap.Height),
    TBitmapLayer(ChildLayer).Bitmap);
end;

procedure TPictureLayer.RescaleToMaxSizeIfNeed();
var
  TmpBitmap: TBitmap32;
begin
  with TBitmapLayer(ChildLayer) do
  begin
    if (Bitmap.Width > FMaxWidth) or (Bitmap.Height > FMaxHeigth) then
    begin
      TmpBitmap := TBitmap32.Create();
      try
        TmpBitmap.Assign(Bitmap);
        with MakeRect(Location) do
          Bitmap.SetSize(Right - Left, Bottom - Top);
        TmpBitmap.Resampler := TLinearResampler.Create(TmpBitmap);
        TmpBitmap.DrawMode := dmOpaque;
        TmpBitmap.DrawTo(Bitmap, Classes.Rect(0, 0, Bitmap.Width, Bitmap.Height));
      finally
        FreeAndNil(TmpBitmap);
      end;
    end;
  end;
end;

procedure TPictureLayer.SaveBitmapToStream(var ABitmapRect: TRect;
  AStream: TStream);
var ScaleBitmap: TBitmap32;
begin
  with TBitmapLayer(ChildLayer) do
  begin
    ABitmapRect := MakeRect(Location);
    Bitmap.MasterAlpha := $FF;
    if ((ABitmapRect.Right - ABitmapRect.Left) < Bitmap.Width) or
       ((ABitmapRect.Bottom - ABitmapRect.Top) < Bitmap.Height) then
    begin
      // resize the layer's bitmap to the size of the layer
      ScaleBitmap := TBitmap32.Create();
      try
        ScaleBitmap.Assign(Bitmap);
        Bitmap.SetSize(ABitmapRect.Right - ABitmapRect.Left, ABitmapRect.Bottom - ABitmapRect.Top);
        ScaleBitmap.Resampler := TLinearResampler.Create(ScaleBitmap);
        ScaleBitmap.DrawMode := dmOpaque;
        ScaleBitmap.DrawTo(Bitmap, Classes.Rect(0, 0, Bitmap.Width, Bitmap.Height));
      finally
        FreeAndNil(ScaleBitmap);
      end;
    end;
    Bitmap.SaveToStream(AStream);
  end;
end;

procedure TPictureLayer.SetMasterAlpha(const Value: Integer);
begin
  FMasterAlpha := Value;
  TBitmapLayer(ChildLayer).Bitmap.MasterAlpha := Value;
end;

end.
